


/*******************************************************************************
**                             INCLUDE FILES
*******************************************************************************/

#include "drv_includes.h"


/*
*********************************************************************************************************
*                                       installIrq
*
* Description : Install interrupt handler
*
* Arguments   : Interrupt number, interrupt handler address, interrupt priority
*
* Returns     : true or false, return false if IntNum is out of range
*********************************************************************************************************
*/

UINT16 installIrq( UINT16 IntNumber, void *HandlerAddr, UINT16 Priority )
{
  UINT32 *vect_addr;
  UINT32 *vect_cntl;

  VICIntEnClr = 1 << IntNumber;	/* Disable Interrupt */
  VICIntSelect &= ~(1 << IntNumber);      // select irq (not fiq)
  if ( IntNumber >= VIC_SIZE )
  {
    return ( FALSE );
  }
  else
  {
    /* find first un-assigned VIC address for the handler */
    vect_addr = (UINT32 *)(VIC_BASE + VICVectAddr0_OFFSET + IntNumber*4);
    vect_cntl = (UINT32 *)(VIC_BASE + VICVectPriority0_OFFSET + IntNumber*4);
    *vect_addr = (UINT32)HandlerAddr;	/* set interrupt vector */
    *vect_cntl = Priority;
    VICIntEnable = 1 << IntNumber;	/* Enable Interrupt */
    return( TRUE );
  }
}


/*
*********************************************************************************************************
*                                       installFiq
*
* Description : Install fast interrupt handler
*
* Arguments   : Interrupt number
*
* Returns     : none
*********************************************************************************************************
*/


void installFiq( UINT16 IntNumber )
{
  VICIntEnClr = 1 << IntNumber;	/* Disable Interrupt */
  VICIntSelect |= (1 << IntNumber);      // select fiq
  VICIntEnable = 1 << IntNumber;	/* Enable Interrupt */
}



/*
*********************************************************************************************************
*                                       uninstallIrq
*
* Description : Uninstall interrupt handler
*
* Arguments   : Interrupt number
*
* Returns     : none
*********************************************************************************************************
*/

void uninstallIrq( UINT16 IntNumber )
{
  UINT32 *vect_addr;
  UINT32 *vect_cntl;

  VICIntEnClr = 1 << IntNumber;	/* Disable Interrupt */
  /* find first un-assigned VIC address for the handler */
  vect_addr = (UINT32 *)(VIC_BASE + VICVectAddr0_OFFSET + IntNumber*4);
  vect_cntl = (UINT32 *)(VIC_BASE + VICVectPriority0_OFFSET + IntNumber*4);
  *vect_addr = (UINT32)0x00;	/* reset interrupt vector */
  *vect_cntl = LOWEST_PRIORITY;
}


/*
*********************************************************************************************************
*                                       uninstallFiq
*
* Description : UnInstall fast interrupt handler
*
* Arguments   : Interrupt number
*
* Returns     : none
*********************************************************************************************************
*/


void uninstallFiq( UINT16 IntNumber )
{
  VICIntEnClr = 1 << IntNumber;	/* Disable Interrupt */
}
